﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDeckerBus
{
    public class DrawingBus
    {
        public EntityBus Bus { get; protected set; }

        protected float _startPosX;
        protected float _startPosY;
        private int? _pictureWidth = null;
        private int? _pictureHeight = null;
        private readonly int _busWidth = 100;
        private readonly int _busHeight = 30;


        public DrawingBus(int speed, float weight, Color bodyColor) {
            Bus = new EntityBus(speed, weight, bodyColor);
        }

        protected DrawingBus(int speed, float weight, Color bodyColor, int busWidth, int busHeight) : this(speed, weight, bodyColor) 
        {
            _busWidth = busWidth;
            _busHeight = busHeight;
        }

        public void SetPosition(int x, int y, int width, int height) {
            if (_pictureWidth <= _busWidth || _pictureHeight <= _busHeight) {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }

        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                case Direction.Right:
                    if (_startPosX + _busWidth + Bus.Step < _pictureWidth) {
                        _startPosX += Bus.Step;
                    }
                    break;
                case Direction.Left:
                    if (_startPosX - Bus.Step >= 0) {
                        _startPosX -= Bus.Step;
                    }
                    break;
                case Direction.Up:
                    if (_startPosY - Bus.Step >= 0) {
                        _startPosY -= Bus.Step;
                    }
                    break;
                case Direction.Down:
                    if (_startPosY + _busHeight + Bus.Step < _pictureHeight) {
                        _startPosY += Bus.Step;
                    }
                    break;
            }
        }

        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            
            g.DrawRectangle(pen, _startPosX - 1, _startPosY + 11, 100, 30);
            Brush brBodyColor = new SolidBrush(Bus.BodyColor);
            g.FillRectangle(brBodyColor, _startPosX, _startPosY + 10, 100, 30);

            g.DrawRectangle(pen, _startPosX + 30, _startPosY + 20, 10, 20);
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillRectangle(brBlack, _startPosX + 30, _startPosY + 20, 10, 20);

            g.DrawEllipse(pen, _startPosX + 7, _startPosY + 35, 10, 10);
            g.DrawEllipse(pen, _startPosX + 77, _startPosY + 35, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 7, _startPosY + 35, 10, 10);
            g.FillEllipse(brBlack, _startPosX + 77, _startPosY + 35, 10, 10);

            Brush brBlue = new SolidBrush(Color.Blue);
            g.FillEllipse(brBlue, _startPosX + 10, _startPosY + 15, 10, 15);
            g.FillEllipse(brBlue, _startPosX + 50, _startPosY + 15, 10, 15);
            g.FillEllipse(brBlue, _startPosX + 70, _startPosY + 15, 10, 15);
            g.FillEllipse(brBlue, _startPosX + 90, _startPosY + 15, 10, 15);
        }

        public void ChangeBorders(int width, int height) {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _busWidth || _pictureHeight <= _busHeight){ 
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _busWidth > _pictureWidth) {
                _startPosX = _pictureWidth.Value - _busWidth;
            }
            if (_startPosY + _busHeight > _pictureHeight) {
                _startPosY = _pictureHeight.Value - _busHeight;
            }
        }

        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition() {
            return (_startPosX, _startPosX + _busWidth, _startPosY, _startPosY + _busHeight);
        }
    }
}
