﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DoubleDeckerBus
{
    internal class MapWithSetBusesGeneric<T, U>
        where T : class, IDrawingObject
        where U : AbstractMap
    {

        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly int _placeSizeWidth = 210;
        private readonly int _placeSizeHeight = 90;
        private readonly SetBusesGeneric<T> _setBuses;
        private readonly U _map;

        public MapWithSetBusesGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setBuses = new SetBusesGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }

        public static int operator +(MapWithSetBusesGeneric<T, U> map, T bus)
        {   
            return map._setBuses.Insert(bus);
        }

        public static T operator -(MapWithSetBusesGeneric<T, U> map, int position)
        {
            return map._setBuses.Remove(position);
        }
  
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawBuses(gr);
            return bmp;
        }

        public Bitmap ShowOnMap()
        {
            Shaking();
            foreach (var bus in _setBuses.GetBuses())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, bus);
            }
            return new(_pictureWidth, _pictureHeight);
        }

        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }

        private void Shaking()
        {
            int j = _setBuses.Count - 1;
            for (int i = 0; i < _setBuses.Count; i++)
            {
                if (_setBuses[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var bus = _setBuses[j];
                        if (bus != null)
                        {
                            _setBuses.Insert(bus, i);
                            _setBuses.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }

        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            Brush pointColor = new SolidBrush(Color.Orange);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
                    g.DrawLine(pen, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight,
                        i * _placeSizeWidth + _placeSizeWidth / 2, (int)((j * _placeSizeHeight) - (40)));

                    g.FillPolygon(pointColor, new Point[] {
                        new Point(i * _placeSizeWidth + _placeSizeWidth / 2, (int)((j * _placeSizeHeight) - (40))),
                        new Point(i * _placeSizeWidth + _placeSizeWidth / 2 - 10, (int)((j * _placeSizeHeight) - (50))),
                        new Point(i * _placeSizeWidth + _placeSizeWidth / 2, (int)((j * _placeSizeHeight) - (60))),
                        new Point(i * _placeSizeWidth + _placeSizeWidth / 2 + 10, (int)((j * _placeSizeHeight) - (50)))
                    });
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
            }
        }

        private void DrawBuses(Graphics g)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int height = _pictureHeight / _placeSizeHeight;

            int currentWidth = width - 1;
            int currentHeight = 0;

            foreach (var bus in _setBuses.GetBuses())
            {

                bus?.SetObject(currentWidth * _placeSizeWidth,
                    currentHeight * _placeSizeHeight,
                    _pictureWidth, _pictureHeight);
                bus?.DrawingObject(g);

                if (currentWidth > 0)
                    currentWidth--;
                else
                {
                    currentWidth = width - 1;
                    currentHeight++;
                }
                if (currentHeight > height) return;
            }
        }

        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var bus in _setBuses.GetBuses())
            {
                data += $"{bus.GetInfo()}{separatorData}";
            }
            return data;
        }

        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setBuses.Insert(DrawingObjectBus.Create(rec) as T);
            }
        }
    }
}
